# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: model.j2

# Fleet Commander (1.7.1)

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from accelbyte_py_sdk.core import Model


class TimeZone(Model):
    """Time zone (time.zone)

    Properties:
        is_dst: (isDST) REQUIRED bool

        name: (name) REQUIRED str

        offset: (offset) REQUIRED int
    """

    # region fields

    is_dst: bool  # REQUIRED
    name: str  # REQUIRED
    offset: int  # REQUIRED

    # endregion fields

    # region with_x methods

    def with_is_dst(self, value: bool) -> TimeZone:
        self.is_dst = value
        return self

    def with_name(self, value: str) -> TimeZone:
        self.name = value
        return self

    def with_offset(self, value: int) -> TimeZone:
        self.offset = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "is_dst"):
            result["isDST"] = bool(self.is_dst)
        elif include_empty:
            result["isDST"] = False
        if hasattr(self, "name"):
            result["name"] = str(self.name)
        elif include_empty:
            result["name"] = ""
        if hasattr(self, "offset"):
            result["offset"] = int(self.offset)
        elif include_empty:
            result["offset"] = 0
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(cls, is_dst: bool, name: str, offset: int, **kwargs) -> TimeZone:
        instance = cls()
        instance.is_dst = is_dst
        instance.name = name
        instance.offset = offset
        return instance

    @classmethod
    def create_from_dict(cls, dict_: dict, include_empty: bool = False) -> TimeZone:
        instance = cls()
        if not dict_:
            return instance
        if "isDST" in dict_ and dict_["isDST"] is not None:
            instance.is_dst = bool(dict_["isDST"])
        elif include_empty:
            instance.is_dst = False
        if "name" in dict_ and dict_["name"] is not None:
            instance.name = str(dict_["name"])
        elif include_empty:
            instance.name = ""
        if "offset" in dict_ and dict_["offset"] is not None:
            instance.offset = int(dict_["offset"])
        elif include_empty:
            instance.offset = 0
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, TimeZone]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[TimeZone]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[TimeZone, List[TimeZone], Dict[Any, TimeZone]]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "isDST": "is_dst",
            "name": "name",
            "offset": "offset",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "isDST": True,
            "name": True,
            "offset": True,
        }

    # endregion static methods
